/*******************************************************************************

$Id: semaphore.h,v 1.2 1998/01/20 13:33:16 timmy Exp $

semaphore.h
libmoreposix for BeOS
Copyright 1997 R3 Software Ltd.
Author: Timothy Wayper
Free for use to a good home.

Notes on usage:
Provides POSIX 1003.1 and 1003.1b-1993 support for semaphores.
Helpful for writting slightly more portable code or to port existing POSIX apps
which use these functions. New code for BeOS will probably use the BeOS kernel
directly (see <OS.h>) as it's nicer to use: cleaner and more powerful.

You will want to add the following preprocessor symbol to <unistd.h> in order
to use this file: #define _POSIX_SEMAPHORES
And to <limits.be.h>:
#define SEM_NSEMS_MAX 256 // you can increase this one if you need to
#define SEM_VALUE_MAX 2147483648
You may also have to mess around with checks for the _POSIX_VERSION symbol.

Caveats:
- named semaphores are persistant, but they will not work across a network.
- named semaphore operations require the moreposix_server to be running
- sem_unlink() is not supported yet (returns -1 with errno set to ENOSYS)

Questions about this file or bug reports should be directed to:
	<timmy@r3.co.nz>

Legal Stuff: This may work, it may not. Use at your own risk.

*******************************************************************************/

#ifndef _SEMAPHORE_H
#define _SEMAPHORE_H

#ifdef __BEOS__
	#include <OS.h>
#else
	#error requires BeOS to compile!
#endif

/* types */

typedef struct sem_t { /* NOTE: declared for size, but treat as opaque! */
	uint8 _pshared; /* shared state flag */
	char _name[NAME_MAX - 7]; /* internal name */
	sem_id _id; /* BeOS kernel sem_id */
	uint32 _link; /* internal link reference count */
} sem_t;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef __MWERKS__
	#pragma export on
#endif

/* prototypes */

/* note: named semaphore operations such as sem_close() require the
   moreposix_server to be running */
int sem_close(sem_t *sem);

int sem_destroy(sem_t *sem);

int sem_getvalue(sem_t *sem, int *value);

int sem_init(sem_t *sem, int pshared, unsigned value);

/* note: named semaphore operations such as sem_open() require the
   moreposix_server to be running */
sem_t *sem_open(char *sem_name, int oflags, ... );

int sem_post(sem_t *sem);

int sem_trywait(sem_t *sem);

int sem_unlink(char *sem_name);

int sem_wait(sem_t *sem);

#ifdef __MWERKS__
	#pragma export reset
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

/******************************************************************************/

#endif /* _SEMAPHORE_H */

/* end semaphore.h */